program SceneTest;

uses
  Windows,
  SysUtils,
  Classes,
  GGraphic,
  DemoTools,
  GfxWin,
  RenD3D,
  RenOGL,
  WinUtils,
  Messages,
  INIFiles,
  Images,
  ImgJPG,
  DynLights,
  Lights,
  GfxTypes,
  WindowCls,
  Colors,
  GeoObjs,
  GeoExt,
  Types3D,
  Utils3D,
  Textures,
  SceneEngine,
  Fog,
  Sky,
  SkyDome,
  SkyBase,
  GScene,
  GFileIO,
  Variants,
  Meshes,
  Cameras,
  Objs3D,
  SceneTypes,
  SceneUpdate;

var
  Win: TGfxWin;
  FPSTime: DWord;
  InputExclusive: boolean = not true;
  Lighting: boolean = not true;
  WireFrame: boolean = false;
  Help: boolean = false;
  BackColor: TARGB = $FF112244;
 // move speed in centimeters per second
  Speed: single = 200;
  RunSpeed: single = 400;
  // options read from inifile
  IniWidth: Longint;
  IniHeight: LongInt;
  IniBits: LongInt;
  IniFull: boolean;
  IniMultiSamples: LongInt;
  IniHardwareTL: boolean;
  IniRenderer: string;
  IniTexBits: LongInt;
  IniTexDetail: string;
  INI: TIniFile;

 // world
  Grass: TGeoQuad;
  BaseTower1: TCylinder;
  BaseTower2: TCylinder;
  UpTower1: TCylinder;
  UpTower2: TCylinder;
  TopRoom: TGeoBox;
  PilarBase1: TCylinder;
  Pilar1: TCylinder;
  PilarBase2: TCylinder;
  Pilar2: TCylinder;
  Entry: TGeoBox;
  Door: TGeoQuad;
  EntryRoof: TCylinder;
  Stick1: TCylinder;
  Stick2: TCylinder;
  Flag1: TGeoQuad;
  Flag2: TGeoQuad;
  BaseTower3: TCylinder;
  BaseTower4: TCylinder;
  UpTower3: TCylinder;
  UpTower4: TCylinder;
  TopRoom2: TGeoBox;
  PilarBase3: TCylinder;
  Pilar3: TCylinder;
  PilarBase4: TCylinder;
  Pilar4: TCylinder;
  Stick3: TCylinder;
  Stick4: TCylinder;
  Flag3: TGeoQuad;
  Flag4: TGeoQuad;
  Wall1: TGeoBox;
  Wall2: TGeoBox;
  TopRoom3: TGeoBox;
  TopRoom4: TGeoBox;
  EntryLit: TDynamicLight;
  EntBox: TGeoBox;
  Ebx:TGeoBox;

  Castle2:TBase3DObj;
  Castle3:TBase3DObj;
  Castle4:TBase3DObj;

  Castle5:TBase3DObj;
  Castle6:TBase3DObj;
  Castle7:TBase3DObj;
  Castle8:TBase3DObj;
  Castle9:TBase3DObj;

  sp: TSphere;
  Lst: dword;

{$R *.res}

procedure OnDraw; forward;
procedure CheckInput; forward;
procedure OnMessage(var Msg: TMessage); forward;
procedure OnRun; forward;

procedure CreateWorld;
begin
  FPSTime := GetTickCount;
  Graphic.AutoClearTarget := false;
  Graphic.ClearTarget($FF000000);
  Graphic.RenderTexture(Win.ClientWidth - 210, Win.ClientHeight - 60,
    TexTureMgr.GetPicture('vamptech.tga'));
  Graphic.RenderTexture(50, (Win.ClientHeight - 100) div 2 + 100,
    TexTureMgr.GetPicture('loading.tga'));
  Graphic.RenderTexture(80, (Win.ClientHeight - 100) div 2 - 80,
    TexTureMgr.GetPicture('swordsman.tga'));

  Graphic.Present;
  Graphic.AutoClearTarget := true;
  Grass := TGeoQuad.Create(20000,20000 , 0, TextureMgr.GetTexture('grass.jpg'));
  Grass.MoveByAxes(0, 0, 100);
  Grass.TexMapper.SetTiling(20, 20);
  Grass.Init;
 // Scene.Add(Grass);

  BaseTower1 := TCylinder.Create(500, 1200, TextureMgr.GetTexture('Wall.jpg'),
    32); ;
  BaseTower1.MoveByAxes(1500, 2000, 700);
  BaseTower1.TopCap := ctAntiCenter;
  BaseTower1.BottomCap := ctAntiCenter;
  BaseTower1.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  BaseTower1.TexMappers[spMain].SetTiling(4, 2);
  BaseTower1.CapCenterUnitYCoord := true;
  BaseTower1.AntiCenterLength := 400;
  BaseTower1.RotateByAxes(0, 0, 180);
  BaseTower1.VisibleParts[spBottomCap] := false;
  BaseTower1.Init;
  Scene.Add(BaseTower1);


  BaseTower2 := TCylinder.Create(500, 1200, TextureMgr.GetTexture('Wall.jpg'),
    32); ;
  BaseTower2.MoveByAxes(3500, 2000, 700);
  BaseTower2.TopCap := ctAntiCenter;
  BaseTower2.BottomCap := ctAntiCenter;
  BaseTower2.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  BaseTower2.TexMappers[spMain].SetTiling(4, 2);
  BaseTower1.CapCenterUnitYCoord := true;
  BaseTower2.AntiCenterLength := 400;
  BaseTower2.VisibleParts[spBottomCap] := false;
  BaseTower2.Init;
  Scene.Add(BaseTower2);


  UpTower1 := TCylinder.Create(350, 800, TextureMgr.GetTexture('Wall.jpg'), 18);
  ;
  UpTower1.MoveByAxes(1900, 2000, 1510);
  UpTower1.TexMappers[spMain].SetTiling(3.5, 2);
  UpTower1.TopCap := ctAntiCenter;
  UpTower1.BottomCap := ctAntiCenter;
  UpTower1.CapCenterUnitYCoord := true;
  UpTower1.Textures[spBottomCap] := TextureMgr.GetTexture('Wood2.jpg');
  UpTower1.TexMappers[spBottomCap].SetTiling(6, 2);
  UpTower1.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  UpTower1.AntiCenterLength := 400;
  UpTower1.Init;
  Scene.Add(UpTower1);


  UpTower2 := TCylinder.Create(350, 800, TextureMgr.GetTexture('Wall.jpg'), 18);
  UpTower2.MoveByAxes(3100, 2000, 1510);
  UpTower2.TexMappers[spMain].SetTiling(3.5, 2);
  UpTower2.TopCap := ctAntiCenter;
  UpTower2.BottomCap := ctAntiCenter;
  UpTower2.Textures[spBottomCap] := TextureMgr.GetTexture('Wood2.jpg');
  UpTower2.CapCenterUnitYCoord := true;
  UpTower2.TexMappers[spBottomCap].SetTiling(6, 2);
  UpTower2.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  UpTower2.RotateByAxes(0, 0, 180);
  UpTower2.AntiCenterLength := 400;
  UpTower2.Init;
  Scene.Add(UpTower2);


  TopRoom := TGeoBox.Create(700, 500, 500,
    TextureMgr.GetTexture('WoodWall.jpg')); ;
  TopRoom.MoveByAxes(2500, 2000, 1500);
  TopRoom.Init;
  Scene.Add(TopRoom);


  PilarBase1 := TCylinder.Create(65, 50, TextureMgr.GetTexture('Stone2.jpg'),
    16); ;
  PilarBase1.MoveByAxes(2250, 2000, 125);
  PilarBase1.TexMappers[spMain].SetTiling(2, 1);
  PilarBase1.Textures[spTopCap] := TextureMgr.GetTexture('marble.jpg');
  PilarBase1.VisibleParts[spBottomCap] := false;
  PilarBase1.Init;
  Scene.Add(PilarBase1);


  PilarBase2 := TCylinder.Create(65, 50, TextureMgr.GetTexture('Stone2.jpg'),
    16); ;
  PilarBase2.MoveByAxes(2750, 2000, 125);
  PilarBase2.TexMappers[spMain].SetTiling(2, 1);
  PilarBase2.Textures[spTopCap] := TextureMgr.GetTexture('marble.jpg');
  PilarBase2.VisibleParts[spBottomCap] := false;
  PilarBase2.Init;
  Scene.Add(PilarBase2);


  Pilar1 := TCylinder.Create(45, 1100, TextureMgr.GetTexture('wall3.jpg'), 32);
  Pilar1.MoveByAxes(2250, 2000, 700);
  Pilar1.TexMappers[spMain].SetTiling(1, 5);
  Pilar1.VisibleParts[spTopCap] := false;
  Pilar1.VisibleParts[spBottomCap] := false;
  Pilar1.Init;
  Scene.Add(Pilar1);


  Pilar2 := TCylinder.Create(45, 1100, TextureMgr.GetTexture('wall3.jpg'), 32);
  Pilar2.MoveByAxes(2750, 2000, 700);
  Pilar2.TexMappers[spMain].SetTiling(1, 5);
  Pilar2.VisibleParts[spTopCap] := false;
  Pilar2.VisibleParts[spBottomCap] := false;
  Pilar2.Init;
  Scene.Add(Pilar2);


  Stick1 := TCylinder.Create(5, 600, TextureMgr.GetTexture('wood2.jpg'), 4);
  Stick1.MoveByAxes(1900, 2000, 2600);
  Stick1.Init;
  Scene.Add(Stick1);


  Stick2 := TCylinder.Create(5, 600, TextureMgr.GetTexture('wood2.jpg'), 4);
  Stick2.MoveByAxes(3100, 2000, 2600);
  Stick2.Init;
  Scene.Add(Stick2);


  Flag1 := TGeoQuad.Create(400, 200, 00, TextureMgr.GetTexture('flag.tga'));
  Flag1.MoveByAxes(1700, 2000, 2780);
  Flag1.RotateByAxes(90, 0, 180);
  Flag1.TexMapper.SetClamp(true, true);
  Flag1.TexMapper.SetClampValues(0.01, 0.01, 0.98, 0.99);
  Flag1.Culled := false;
  Flag1.UseDeviceBuffer := false;
  Flag1.Init;
  Scene.Add(Flag1);


  Flag2 := TGeoQuad.Create(400, 200, 00, TextureMgr.GetTexture('flag.tga'));
  Flag2.MoveByAxes(2900, 2000, 2780);
  Flag2.RotateByAxes(90, 0, 180);
  Flag2.TexMapper.SetClamp(true, true);
  Flag2.TexMapper.SetClampValues(0.01, 0.01, 0.98, 0.99);
  Flag2.Culled := false;
  Flag2.UseDeviceBuffer := false;
  Flag2.Init;
  Scene.Add(Flag2);



  BaseTower3 := TCylinder.Create(500, 1200, TextureMgr.GetTexture('Wall.jpg'),
    24); ;
  BaseTower3.MoveByAxes(1500, 3500, 700);
  BaseTower3.TopCap := ctAntiCenter;
  BaseTower3.BottomCap := ctAntiCenter;
  BaseTower3.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  BaseTower3.TexMappers[spMain].SetTiling(4, 2);
  BaseTower3.CapCenterUnitYCoord := true;
  BaseTower3.AntiCenterLength := 400;
  BaseTower3.RotateByAxes(0, 0, 180);
  BaseTower3.VisibleParts[spBottomCap] := false;
  BaseTower3.Init;
  Scene.Add(BaseTower3);


  BaseTower4 := TCylinder.Create(500, 1200, TextureMgr.GetTexture('Wall.jpg'),
    24); ;
  BaseTower4.MoveByAxes(3500, 3500, 700);
  BaseTower4.TopCap := ctAntiCenter;
  BaseTower4.BottomCap := ctAntiCenter;
  BaseTower4.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  BaseTower4.TexMappers[spMain].SetTiling(4, 2);
  BaseTower4.CapCenterUnitYCoord := true;
  BaseTower4.AntiCenterLength := 400;
  BaseTower4.VisibleParts[spBottomCap] := false;
  BaseTower4.Init;
  Scene.Add(BaseTower4);


  UpTower3 := TCylinder.Create(350, 800, TextureMgr.GetTexture('Wall.jpg'), 18);
  ;
  UpTower3.MoveByAxes(1900, 3500, 1510);
  UpTower3.TexMappers[spMain].SetTiling(3.5, 2);
  UpTower3.TopCap := ctAntiCenter;
  UpTower3.BottomCap := ctAntiCenter;
  UpTower3.CapCenterUnitYCoord := true;
  UpTower3.Textures[spBottomCap] := TextureMgr.GetTexture('Wood2.jpg');
  UpTower3.TexMappers[spBottomCap].SetTiling(6, 2);
  UpTower3.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  UpTower3.AntiCenterLength := 400;
  UpTower3.Init;
  Scene.Add(UpTower3);


  UpTower4 := TCylinder.Create(350, 800, TextureMgr.GetTexture('Wall.jpg'), 18);
  UpTower4.MoveByAxes(3100, 3500, 1510);
  UpTower4.TexMappers[spMain].SetTiling(3.5, 2);
  UpTower4.TopCap := ctAntiCenter;
  UpTower4.BottomCap := ctAntiCenter;
  UpTower4.Textures[spBottomCap] := TextureMgr.GetTexture('Wood2.jpg');
  UpTower4.CapCenterUnitYCoord := true;
  UpTower4.TexMappers[spBottomCap].SetTiling(6, 2);
  UpTower4.Textures[spTopCap] := TextureMgr.GetTexture('TowerRoof.jpg');
  UpTower4.RotateByAxes(0, 0, 180);
  UpTower4.AntiCenterLength := 400;
  UpTower4.Init;
  Scene.Add(UpTower4);


  TopRoom2 := TGeoBox.Create(700, 500, 500,
    TextureMgr.GetTexture('WoodWall.jpg')); ;
  TopRoom2.MoveByAxes(2500, 3500, 1500);
  TopRoom2.Init;
  Scene.Add(TopRoom2);


  PilarBase3 := TCylinder.Create(65, 50, TextureMgr.GetTexture('Stone2.jpg'),
    16); ;
  PilarBase3.MoveByAxes(2250, 3500, 125);
  PilarBase3.TexMappers[spMain].SetTiling(2, 1);
  PilarBase3.Textures[spTopCap] := TextureMgr.GetTexture('marble.jpg');
  PilarBase3.VisibleParts[spBottomCap] := false;
  PilarBase3.Init;
  Scene.Add(PilarBase3);


  PilarBase4 := TCylinder.Create(65, 50, TextureMgr.GetTexture('Stone2.jpg'),
    16); ;
  PilarBase4.MoveByAxes(2750, 3500, 125);
  PilarBase4.TexMappers[spMain].SetTiling(2, 1);
  PilarBase4.Textures[spTopCap] := TextureMgr.GetTexture('marble.jpg');
  PilarBase4.VisibleParts[spBottomCap] := false;
  PilarBase4.Init;
  Scene.Add(PilarBase4);


  Pilar3 := TCylinder.Create(45, 1100, TextureMgr.GetTexture('wall3.jpg'), 32);
  Pilar3.MoveByAxes(2250, 3500, 700);
  Pilar3.TexMappers[spMain].SetTiling(1, 5);
  Pilar3.VisibleParts[spTopCap] := false;
  Pilar3.VisibleParts[spBottomCap] := false;
  Pilar3.Init;
  Scene.Add(Pilar3);


  Pilar4 := TCylinder.Create(45, 1100, TextureMgr.GetTexture('wall3.jpg'), 32);
  Pilar4.MoveByAxes(2750, 3500, 700);
  Pilar4.TexMappers[spMain].SetTiling(1, 5);
  Pilar4.VisibleParts[spTopCap] := false;
  Pilar4.VisibleParts[spBottomCap] := false;
  Pilar4.Init;
  Scene.Add(Pilar4);


  Stick3 := TCylinder.Create(5, 600, TextureMgr.GetTexture('wood2.jpg'), 4);
  Stick3.MoveByAxes(1900, 3500, 2600);
  Stick3.Init;
  Scene.Add(Stick3);


  Stick4 := TCylinder.Create(5, 600, TextureMgr.GetTexture('wood2.jpg'), 4);
  Stick4.MoveByAxes(3100, 3500, 2600);
  Stick4.Init;
  Scene.Add(Stick4);


  Flag3 := TGeoQuad.Create(400, 200, 00, TextureMgr.GetTexture('flag.tga'));
  Flag3.MoveByAxes(1700, 3500, 2780);
  Flag3.RotateByAxes(90, 0, 180);
  Flag3.TexMapper.SetClamp(true, true);
  Flag3.TexMapper.SetClampValues(0.01, 0.01, 0.98, 0.99);
  Flag3.Culled := false;
  Flag3.UseDeviceBuffer := false;
  Flag3.Init;
  Scene.Add(Flag3);


  Flag4 := TGeoQuad.Create(400, 200, 00, TextureMgr.GetTexture('flag.tga'));
  Flag4.MoveByAxes(2900, 3500, 2780);
  Flag4.RotateByAxes(90, 0, 180);
  Flag4.TexMapper.SetClamp(true, true);
  Flag4.TexMapper.SetClampValues(0.01, 0.01, 0.98, 0.99);
  Flag4.Culled := false;
  Flag4.UseDeviceBuffer := false;
  Flag4.Init;
  Scene.Add(Flag4);



  Entry := TGeoBox.Create(300, 300, 250,
    TextureMgr.GetTexture('Wall_.jpg')); ;
  Entry.MoveByAxes(2500, 2750, 225);
  Entry.SetTexTiling(1.5, 1.5);
  Entry.Init;
  Scene.Add(Entry);

  EntBox := TGeoBox.Create(350, 350, 250,
    TextureMgr.GetTexture('Wall_.jpg')); ;
  Scene.Add(EntBox, Entry);
  EntBox.MoveByAxes(0, 0, 375);
  EntBox.SetTexTiling(1.75, 1.5);
  EntBox.Init;

  EBx := TGeoBox.Create(400, 400, 250,
    TextureMgr.GetTexture('Wall_.jpg')); ;
  Scene.Add(EBx, Entry);
  EBx.MoveByAxes(0, 0, 750);
  EBx.SetTexTiling(2, 1.5);
  EBx.Init;

  Door := TGeoQuad.Create(100, 200, 00, TextureMgr.GetTexture('door.jpg'));
  Scene.Add(Door, Entry);
  Door.MoveByAxes(0, -152, -25);
  Door.RotateByAxes(-270, 0, 0);
  Door.Init;


  EntryRoof := TCylinder.Create(150, 2500,
    TextureMgr.GetTexture('wood2.jpg'), 4); ;
  Scene.Add(EntryRoof, Entry);
  EntryRoof.MoveByAxes(0, 0, 1350);
  EntryRoof.TopCap := ctAntiCenter;
  EntryRoof.AntiCenterLength := 600;
  EntryRoof.CapCenterUnitYCoord := true;
  EntryRoof.Init;


  TopRoom3 := TGeoBox.Create(980, 500, 500,
    TextureMgr.GetTexture('WoodWall.jpg')); ;
  TopRoom3.MoveByAxes(1900, 2750, 1500);
  TopRoom3.RotateByAxes(0, 0, 90);
  TopRoom3.Init;
  Scene.Add(TopRoom3);


  TopRoom4 := TGeoBox.Create(980, 500, 500,
    TextureMgr.GetTexture('WoodWall.jpg')); ;
  TopRoom4.MoveByAxes(3100, 2750, 1500);
  TopRoom4.RotateByAxes(0, 0, 90);
  TopRoom4.Init;
  Scene.Add(TopRoom4);


  Wall1 := TGeoBox.Create(500, 50, 350,
    TextureMgr.GetTexture('Wall.jpg')); ;
  Wall1.MoveByAxes(1500, 2750, 275);
  Wall1.SetTexTiling(1, 0.8);
  Wall1.RotateByAxes(0, 0, 90);
  Wall1.TexMappers[bsTop].SetTiling(1, 0.1);
  Wall1.Init;
  Scene.Add(Wall1);


  Wall2 := TGeoBox.Create(500, 50, 350,
    TextureMgr.GetTexture('Wall.jpg')); ;
  Wall2.MoveByAxes(3500, 2750, 275);
  Wall2.SetTexTiling(1, 0.8);
  Wall2.RotateByAxes(0, 0, 90);
  Wall2.TexMappers[bsTop].SetTiling(1, 0.1);
  Wall2.Init;
  Scene.Add(Wall2);


  EntryLit := TDynamicLight.Create(ltPoint, 10000);
  EntryLit.MoveByAxes(2500, 2750, 600);
  EntryLit.DiffuseColor := LightColor(100, 100, -100);
  EntryLit.QuadraticAttenuation := 1 / 100000;
  EntryLit.ConstantAttenuation := 0;
  EntryLit.Init;
  Scene.Add(EntryLit);



  Scene.Sky.SkyPlane.SetTextures(TextureMgr.GetTexture('sky.tga'),
    TextureMgr.GetTexture('grass.jpg'), TextureMgr.GetTexture('skyboxwot.tga'),
    TextureMgr.GetTexture('skyboxwot.tga'),
    TextureMgr.GetTexture('skyboxwot.tga'),
    TextureMgr.GetTexture('skyboxwot.tga')
    );
  Scene.Sky.SkyMode := smSkyNone;
  Scene.Sky.SkyPlane.Height := SkySize / 3;
  SkyPlane.TopSize := SkySize * 3;
  SkyPlane.TopHorizonColor := $00AA5500;
  SkyPlane.BottomHorizonColor := $FFAA0000;

  SkyPlane.CoordYSpeed := 0.007;
 // SkyPlane.UseHorizon:=true;
 // SkyPlane.UseHorizon:=not true;
  SkyPlane.Sides[bsBottom].TexMapper.SetTiling(128,128);
  SkyPlane.Sides[bsBottom].UpdateTexCoords;

 // SkyPLane.Height:=SkySize/3;
 // SkyPlane.TopHeight:=10;
  SkyPlane.ViewerHeight := 0.5;
{  Scene.Sky.SkyBox.Sides[bsTop].Points[0]:=Point3S(-SkySize*3,-SkySize*3,SkySize/2);
  Scene.Sky.SkyBox.Sides[bsTop].Points[1]:=Point3S(SkySize*3,-SkySize*3,SkySize/2);
  Scene.Sky.SkyBox.Sides[bsTop].Points[2]:=Point3S(-SkySize*3,SkySize*3,SkySize/2);
  Scene.Sky.SkyBox.Sides[bsTop].Points[3]:=Point3S(SkySize*3,SkySize*3,SkySize/2);

  Scene.Sky.SkyBox.Sides[bsTop].CoordySpeed:=0.01;
  Scene.Sky.SkyBox.Sides[bsTop].TexMapper.SetClamp(false,false);
  Scene.Sky.SkyBox.Sides[bsTop].UpdateTexCoords;
}
{  SP := TSphere.Create(300, nil, 120);
  Sp.Color := $FFFF0000;
  Sp.MoveAbs(200, 200, 400);
  Sp.Scale := Point3S(3, 2, 1);
  Sp.Init;
  Scene.Add(sp);
}  Castle2:=Scene.Root.Clone;
  Scene.Root.AddChild(Castle2);
  Castle2.MoveAbs(3500,0,0);
  Castle2.Init;

  Castle3:=Castle2.Clone;
  Scene.Root.AddChild(Castle3);
  Castle3.MoveAbs(0,3500,0);
  Castle3.Init;

  Castle4:=Castle2.Clone;
  Scene.Root.AddChild(Castle4);
  Castle4.MoveAbs(3500,3500,0);
  Castle4.Init;

  Castle5:=Castle2.Clone;
  Scene.Root.AddChild(Castle5);
  Castle5.MoveAbs(7000,3500,0);
  Castle5.Init;

  Castle6:=Castle2.Clone;
  Scene.Root.AddChild(Castle6);
  Castle6.MoveAbs(7000,0,0);
  Castle6.Init;

  Castle7:=Castle2.Clone;
  Scene.Root.AddChild(Castle7);
  Castle7.MoveAbs(7000,7000,0);
  Castle7.Init;

  Castle8:=Castle2.Clone;
  Scene.Root.AddChild(Castle8);
  Castle8.MoveAbs(0,7000,0);
  Castle8.Init;

  Castle9:=Castle2.Clone;
  Scene.Root.AddChild(Castle9);
  Castle9.MoveAbs(3500,7000,0);
  Castle9.Init;



  Camera.MoveByAxes(1500, 500, 260);
  Camera.LookAt(Door);
  SetCursorPos(100, 100);
  InputExclusive := true;

  Graphic.ClearColor:=BackColor;
//  Graphic.AutoClearTarget := false;
//  SetPriorityClass(GetCurrentProcess,HIGH_PRIORITY_CLASS	);
end;

procedure OnFirstDraw;
begin

end;


procedure OnDraw;
var I: LongInt;
begin
  CheckInput;

  Win.Caption := Format('X:%f  Y:%f  Z:%f', [camera.position.x,
    camera.position.y, camera.position.z]);

  {Graphic.Disable(gfFog);
  Grass.DrawAlone;
  Graphic.Enable(gfFog);
   }
  EntryLit.UpdateLight;

  for I := 0 to 0 do
  begin
  {BaseTower1.DrawAlone;
  BaseTower2.DrawAlone;
  TopRoom.DrawAlone;
  UpTower1.DrawAlone;
  UpTower2.DrawAlone;
  PilarBase1.DrawAlone;
  PilarBase2.DrawAlone;
  Pilar1.DrawAlone;
  Pilar2.DrawAlone;
  Entry.DrawAlone;
  EntryRoof.DrawAlone;
  Door.DrawAlone;
  Stick1.DrawAlone;
  Stick2.DrawAlone;
  BaseTower3.DrawAlone;
  BaseTower4.DrawAlone;
  TopRoom2.DrawAlone;
  UpTower3.DrawAlone;
  UpTower4.DrawAlone;
  PilarBase3.DrawAlone;
  PilarBase4.DrawAlone;
  Pilar3.DrawAlone;
  Pilar4.DrawAlone;
  Stick3.DrawAlone;
  Stick4.DrawAlone;
  TopRoom3.DrawAlone;
  TopRoom4.DrawAlone;
  Wall1.DrawAlone;
  Wall2.DrawAlone;}

//  Scene.TryUpdate;//(GettickCount-lst);
//  lst:=GetTickCount;
 // Scene.Draw;
  end;

//  sp.DrawAlone;
  // simple flag animations
  Flag1.Points[1].Y := Flag1.Points[1].Y + Sin(GetTickCount / 500) / 2;
  Flag1.Points[1].Z := Flag1.Points[1].Z + Cos(GetTickCount / 500);

  Flag1.Points[3].Y := Flag1.Points[3].Y + Sin(GetTickCount / 500) / 2;
  Flag1.Points[3].Z := Flag1.Points[3].Z + Cos(GetTickCount / 400);

  Flag1.UpdateBuffer;

  Flag2.Points[1].Y := Flag2.Points[1].Y + Cos(GetTickCount / 500) / 2;
  Flag2.Points[1].Z := Flag2.Points[1].Z + Sin(GetTickCount / 500);

  Flag2.Points[3].Y := Flag2.Points[3].Y + Cos(GetTickCount / 500) / 2;
  Flag2.Points[3].Z := Flag2.Points[3].Z + Sin(GetTickCount / 400);

  Flag2.UpdateBuffer;


 // Flag1.DrawAlone;
 // Flag2.DrawAlone;


  // simple flag animations
  Flag4.Points[1].Y := Flag4.Points[1].Y + Sin(GetTickCount / 500) / 2;
  Flag4.Points[1].Z := Flag4.Points[1].Z + Cos(GetTickCount / 500);

  Flag4.Points[3].Y := Flag4.Points[3].Y + Sin(GetTickCount / 500) / 2;
  Flag4.Points[3].Z := Flag4.Points[3].Z + Cos(GetTickCount / 400);

  Flag4.UpdateBuffer;

  Flag3.Points[1].Y := Flag3.Points[1].Y + Cos(GetTickCount / 500) / 2;
  Flag3.Points[1].Z := Flag3.Points[1].Z + Sin(GetTickCount / 500);

  Flag3.Points[3].Y := Flag3.Points[3].Y + Cos(GetTickCount / 500) / 2;
  Flag3.Points[3].Z := Flag3.Points[3].Z + Sin(GetTickCount / 400);

  Flag3.UpdateBuffer;


//  Flag3.DrawAlone;
//  Flag4.DrawAlone;



  Graphic.PushDWords;
  Renderer.SetDWord(sdFillMode, FILL_SOLID);
  Graphic.RenderTexture(Win.ClientWidth - 210, Win.ClientHeight - 60,
    TexTureMgr.GetPicture('vamptech.tga'));

  if Help then
    Graphic.RenderTexture(10, 40, TexTureMgr.GetPicture('help.tga'));

  if FPSTime + 1000 < GetTickCount then
  begin
    FPSTime := GetTickCount;
    ImgText.SetCaption(Format('FPS: %f', [Graphic.Fps]));
  end;
  ImgText.Draw;
  Graphic.PopDWords([sdFillMode]);

end;


procedure SetUpGraphic;
begin
  Win := Graphic.Window;
  Win.Caption :=
    'Scene Test  Applycantropion';
  Win.SetClientBounds(50, 20, 700, 500);
  Win.Cursor := crNone;
  Win.Icon.Handle := LoadIcon(HInstance, 'MAINICON');
  Win.OnRunProc := OnRun;
  Graphic.OnDraw := OnDraw;
  Graphic.OnFirstDrawProc := OnFirstDraw;
  Win.OnMsgProc := OnMessage;

  with Graphic do
  begin
    ClearColor := BackColor;
    AutoClearTarget := false;
    Scene.Fog.Enabled := true;
    Scene.Fog.Mode := FOG_LINEAR;
    Scene.Fog.Density:=0.001;
    Scene.Fog.Color := BackColor;
    Scene.Fog.FogStart := 6000;
    Scene.Fog.FogEnd := 6500;

    Renderer.SetDWord(sdAmbient, $FF333333);
    Disable(gfLighting);

    ViewPort.Perspective(45, 20, 7000, 1.33);

    Renderer.SetDWord(sdCullMode, CULL_CW);
    Renderer.SetDWord(sdTexFilter, TEXF_TRILINEAR);

    TextureDetail := 1;

    GfxParams.FullScreen := IniFull;
    GfxParams.DepthBits := 24;
    GfxParams.StencilBits := 0;
    GfxParams.ColorBits := IniBits;
    GfxParams.Width := IniWidth;
    GfxParams.Height := IniHeight;
    GfxParams.UseHardWare := IniHardwareTL;
    GfxParams.MultiSamples := IniMultiSamples;

   Renderer.SetDWord(sdCullMode,CULL_CW);

    TextureBitCount := IniTexBits;

    if UpCase(IniTexDetail[1]) = 'M' then
      TextureDetail := 2
    else
      if UpCase(IniTexDetail[1]) = 'L' then TextureDetail := 4;

  end;


  ImgText.SetFont('Arial', 14, $FFFFFF00);


  Graphic.Reset;
end;

procedure CheckInput;
var Ang: TAngle3S;
  Spd: single;
  dx, dz: single;
  P: TPoint3S;
begin
  if InputExclusive then
  begin
    Input.Update;
    Ang := Camera.Angle;
    dx := Input.GetMouseYRel / 2;
    dz := Input.GetMouseXRel / 2;
    Ang.X := Ang.X - dx;
    Ang.Y := 0;
    Ang.Z := Ang.Z + dz;
    Ang.X := ClampValue(Ang.X, -89, 89);
    Camera.Angle := Ang;
  end;

  if Input.IsKey(VK_ESCAPE) then
    Win.Stop;


  if Input.IsKey(VK_RSHIFT) then
    Spd := RunSpeed
  else
    Spd := Speed;

  Spd := Spd / Max(10, Graphic.FPS);


  if Input.IsKey(VK_LEFT) then
    Camera.SideStep(Spd);
  if Input.IsKey(VK_RIGHT) then
    Camera.SideStep(-Spd);
  if Input.IsKey(VK_UP) then
  begin
    P.x := Sin(DegToRad(180 + Camera.Angle.Z));
    P.y := Cos(DegToRad(Camera.Angle.Z));
    Camera.Position := Point3S(Camera.Position.X + P.X * (Spd),
      Camera.Position.Y + P.Y * (Spd), Camera.Position.Z);
  end;
  if Input.IsKey(VK_DOWN) then
  begin
    P.x := Sin(DegToRad(180 + Camera.Angle.Z));
    P.y := Cos(DegToRad(Camera.Angle.Z));
    Camera.Position := Point3S(Camera.Position.X + P.X * (-Spd),
      Camera.Position.Y + P.Y * (-Spd), Camera.Position.Z);
  end;
  if Input.IsKey(VK_NEXT) then
    Camera.Position := Point3S(Camera.Position.X, Camera.Position.Y,
      Camera.Position.Z - Spd);
  if Input.IsKey(VK_PRIOR) then
    Camera.Position := Point3S(Camera.Position.X, Camera.Position.Y,
      Camera.Position.Z + Spd);


end;

procedure TakeScreenShot;
var Img: TImage;
  I: LongInt;
begin
  JPEGSaveQuality := 95;
  Img := TImage.Create();
  Img.Resize(Win.ClientWidth, Win.ClientHeight);
  Img.BitCount := 24;
  Img.Canvas.CopyRect(Img.Bounds, Win.Canvas, Img.Bounds);
  I := 0;
  while FileExists(ExeDir + 'Shot' + IntToStr(I) + '.jpg') do
    Inc(I);
  Img.Save(ExeDir + 'Shot' + IntToStr(I) + '.jpg');
  Img.Free;



end;


procedure OnMessage(var Msg: TMessage);
var S: tStringList;
  I: LongInt;
  Mat: tMatrix;
begin
  case Msg.Msg of
    WM_KEYDOWN:
      begin

        case Msg.WParam of
          VK_F1: Switch(Help);
          VK_F2:
            begin
              Graphic.GfxParams.RefreshRate := 60;
              Graphic.Reset;
            end;
          VK_F3: Graphic.SetResolution(320, 200, 32, true);
          VK_F4: Graphic.SetResolution(400, 300, 32, true);
          VK_F5: Graphic.SetResolution(640, 400, 32, true);
          VK_F6: Graphic.SetResolution(640, 480, 32, true);
          VK_F7: Graphic.SetResolution(800, 600, 32, true);
          VK_F8: Graphic.SetResolution(1024, 768, 32, true);
          VK_F11: TakeScreenShot;
          Ord('F'):
            begin
              Switch(WireFrame);
              if WireFrame then
                Renderer.SetDWord(sdFillMode, FILL_WIREFRAME)
              else
                Renderer.SetDWord(sdFillMode, FILL_SOLID);
              end;
          Ord('L'):
            begin
              Switch(Lighting);
              if Lighting then
                Graphic.Disable(gfLighting)
              else
                Graphic.Enable(gfLighting);
              EntryLit.UpdateLight;
            end;
          Ord('T'):
            begin
           // Camera.LookAt(entry);
              if not entryroof.InFrustum then
                msgbox('roof not vis');


            end;
               Ord('X'):
            begin
           // Camera.LookAt(entry);
             Castle2.MoveByAxes(20,0,0);

            end;


          Ord('M'):
            begin
              S := tStringList.Create;

            //  Camera.LookAt(0, 0, 0);
              camera.LookAt(wall2);
              Mat := Renderer.GetViewMatrix;
              for I := 0 to 15 do
                S.Add(FloatToStr(Mat.f[I]));
              S.SaveToFile('matview ' + Renderer.Name + '.txt');
              S.Clear;

             Mat:=Renderer.GetprojMatrix;
             for I:=0 to 15 do
              S.Add(FloatToStr(Mat.f[I]));
             S.SaveToFile('matproj '+Renderer.Name+'.txt');
             S.Clear;


             Mat:=MatrixMultiply(Renderer.GetProjMatrix,Renderer.GetViewMatrix);
             for I:=0 to 15 do
              S.Add(FloatToStr(Mat.f[I]));
             S.SaveToFile('mat clip '+Renderer.Name+'.txt');
             S.Clear;


              S.Free;


            end;
            Ord('Q'):
            begin
             Camera.MoveAbs(2500,500,260);
             Camera.LookAt(Door);
             Scene.Enable(soOptimizing);
            end;

            Ord('W'):
            begin
             Camera.MoveAbs(2500,3800,260);
             Camera.LookAt(Flag4);
             Scene.Enable(soOptimizing);
            end;

            Ord('E'):
            begin
             Camera.MoveAbs(2500,500,260);
             Camera.LookAt(Door);
             Scene.Disable(soOptimizing);
            end;



            Ord('R'):
            begin
             Camera.MoveAbs(2500,3800,260);
             Camera.LookAt(Flag4);
             Scene.Disable(soOptimizing);
            end;

            Ord('O'):
            begin
             Switch(Scene.Opts[soOptimizing]);

            end;

        end;
      end;
    WM_RBUTTONDOWN:
      begin
        Switch(InputExclusive);
        if InputExclusive then
          Win.Cursor := crNone
        else
          Win.Cursor := crArrow;
      end;


  end;
end;

procedure OnRun;
begin
  Graphic.Draw;
end;

procedure ReadSettings;
begin
  INI := TINIFile.Create(ExeDir + 'config.ini');
  IniWidth := INI.ReadInteger('Video', 'Width', 800);
  IniHeight := INI.ReadInteger('Video', 'Height', 600);
  IniBits := INI.ReadInteger('Video', 'Bits', 32);
  IniTexBits := INI.ReadInteger('Video', 'TextureBits', 32);
  IniTexDetail := INI.ReadString('Video', 'TextureDetail', 'High');
  IniRenderer := INI.ReadString('Video', 'Renderer', 'Direct3D');
  IniMultiSamples := INI.ReadInteger('Video', 'MultiSamples', 0);
  IniFull := INI.ReadBool('Video', 'FullScreen', false);
  IniHardwareTL := INI.ReadBool('Video', 'HardwareTL', false);

  if UpCase(IniRenderer[1]) = 'D' then
    Graphic.ChangeRenderer(Graphic.GetRenByName('D3DRenderer'), false)
  else
    Graphic.ChangeRenderer(Graphic.GetRenByName('OGLRenderer'), false);


  INI.Free;
end;



begin
  CreateGraphic();
  ReadSettings;
  Graphic.Init;
  Graphic.StopRendering;
  SetUpGraphic;
  CreateWorld;
//  ShutTimer:=TShutTimer.Create(15000);
  //ShutTimer.Start;
 // TShutTimer.Create(15000).Start;



  Win.Run;

  Graphic.Free;


end.

